--new color coding

-- labels
TITCURRTRACK_SHOW_ANNOUNCE = "Show Raid Warning Announcement";
TITCURRTRACK_SHOW_LOG = "Show Announcement in Log";
TITCURRTRACK_SHOW_GLAMOR = "Show Glamor Announcement";
TITCURRTRACK_SHORT_GLAMOR = "Use Small Glamor Announcement";
TITCURRTRACK_SHOW_SUMMARY = "Show Session Summary in Tooltip";
TITCURRTRACK_SHOW_CHANGED = "Color Changed Tokens";
TITCURRTRACK_APPEND_CHANGED = "Append Session Gains";
TITCURRTRACK_SHOW_ALT_SUMMARY = "Show Alt Charaters in Tooltip";

-- globals
TITCURRTRACK_ID =  "CurrTrack";
TITCURRTRACK_VERSION = "1.1.1 for Cataclysm";
TITCURRTRACK_BUTTON_TEXT = "CurrencyTracker";

if(not TitanCurrTrackData) then
	TitanCurrTrackData = {};
end

TitanCurrTrack_cname, TitanCurrTrack_crealm = UnitName("player");
if(not crealm) then
	TitanCurrTrack_crealm = GetRealmName();
end

-- local
local TITCURRTRACK_TOOLTIP_TEXT = "";
local TITCURRTRACK_TTS = {};
local TITCURRTRACK_TIME = GetTime();
local TITCURRTRACK_VALUES = {};
local TITCURRTRACK_VSIZE = 0;


-- initializing
function TitanPanelCurrTrackButton_OnLoad(self)
	self.registry = {
		id = TITCURRTRACK_ID,
		menuText = TITCURRTRACK_ID, 
		version = TITCURRTRACK_VERSION,
		buttonTextFunction = "TitanPanelCurrTrack_GetButtonText", 
		tooltipTitle = "Titan CurrTrack"..TITCURRTRACK_VERSION,
		tooltipTextFunction = "TitanPanelCurrTrack_GetToolTipText", 
		category = "Information",
		savedVariables = {
			ShowAnnounce = nil, 
			ShowGlamorAnnounce = 1, 
			ShortGlamorAnnounce = 1, 
			ShowChanged = 1,
			AppendChanged = 1,
			ShowLog = nil,
			ShowSummary = 1,
			ShowAltSummary = 1,
			TokenHeaders = { "" }
		}
	};	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
end

-- event handling
function TitanPanelCurrTrackButton_OnClick(self, button)
     if (button == "LeftButton") then
	     ToggleCharacter("TokenFrame");
     end
     if (button == "RightButton") then
	     TitanPanelRightClickMenu_PrepareCurrTrackMenu(self);
     end
end

function TitanPanelCurrTrackButton_OnEvent(self, event, ...)	
	TitanPanelButton_UpdateTooltip();
	TitanPanelButton_UpdateButton(TITCURRTRACK_ID);
end

-- for titan to get the displayed text
function TitanPanelCurrTrack_GetButtonText(id)
	TitanPanelCurrTrack_Refresh();
	return TITCURRTRACK_BUTTON_TEXT;
end

function pairsByKeys(t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end


-- for titan tool tip text building
function TitanPanelCurrTrack_GetToolTipText()
	local display="";
	local tooltip="\n";
	local name, isHeader, isExpanded, isUnused, isWatched, count, cCount, PVPIndex;
	cCount = GetCurrencyListSize();
	for index=1, cCount do 
		name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID = GetCurrencyListInfo(index);
		if (isHeader) then
			tooltip = tooltip..TitanUtils_GetHighlightText(name).."\n";
		end
		if (count~=0) then
			display=strconcat(" ",name,"\t",count," |T"..icon..":20|t");
			tooltip=strconcat(tooltip,display,"|r\n");
		end
	end 
	if(TitanGetVar(TITCURRTRACK_ID, "ShowAltSummary") == 1) then
		if(TitanCurrTrackData) then
			tooltip = tooltip.."\n"..TitanUtils_GetHighlightText("Alternate Character Tokens:");
			for crn, cicon in pairsByKeys(TitanCurrTrackData) do
				if(TitanCurrTrack_crealm.."-"..TitanCurrTrack_cname ~= crn) then
					local crealm, cname = strsplit("-",crn);
					if(crealm ~= TitanCurrTrack_crealm) then
						tooltip = tooltip.."\n"..crn.."\t"..cicon;
					else
						tooltip = tooltip.."\n"..cname.."\t"..cicon;
					end
				end
			end
			tooltip = tooltip.."\n";
		end
	end
	if(TitanGetVar(TITCURRTRACK_ID, "ShowSummary") == 1) then
		local timeonline =  GetTime() -	TITCURRTRACK_TIME;
		local humantime = "";

		if (timeonline < 60) then
			humantime = "< 1 Mn";
		else 
			humantime = floor(timeonline / 60);
			if(humantime < 60) then
				humantime = humantime.." Mn";
			else
				local hours = floor(humantime / 60);
				local mins = floor((timeonline - (hours * 60 * 60)) / 60);
				humantime = hours.." Hr "..mins.." Mn";
			end
		end
		tooltip = tooltip.."\n"..TitanUtils_GetHighlightText("Session Summary:");
		tooltip = tooltip.."\t"..TitanUtils_GetNormalText("Duration: "..humantime);

		for f, v in pairs(TITCURRTRACK_TTS) do
			local TPH = floor(v / (timeonline / 60 / 60));
			if(TPH > 0) then
				tooltip = tooltip.."\n "..f.." : "..TitanUtils_GetGreenText(TPH).." perHour\t Total: "..TitanUtils_GetGreenText(v);
			else
				tooltip = tooltip.."\n "..f.." : "..TitanUtils_GetRedText(TPH).." perHour\t Total: "..TitanUtils_GetRedText(v);
			end
		end
	end
	
	final_tooltip=tooltip;
	return ""..final_tooltip;    	
end

function TitanPanelRightClickMenu_AddTitle2(title, level)
	if (title) then
		local info = {};
		info.text = title;
		info.notClickable = 1;
		info.isTitle = 1;
       		info.notCheckable = true;
		UIDropDownMenu_AddButton(info, level);
	end
end


function TitanPanelRightClickMenu_AddToggleVar2(text, id, var, toggleTable)
	local info = {};
	info.text = text;
	info.value = {id, var, toggleTable};
	info.func = function()
		TitanPanelRightClickMenu_ToggleVar({id, var, toggleTable})
	end
	info.checked = TitanGetVar(id, var);
	info.keepShownOnClick = 1;
	UIDropDownMenu_AddButton(info, 2);
end

function TitanPanelRightClickMenu_AddToggleIcon2(id)
	TitanPanelRightClickMenu_AddToggleVar2("Show Icon", id, "ShowIcon");
end

function TitanPanelRightClickMenu_AddSpacer2(level)
	local info = {};
	info.disabled = 1;
       	info.notCheckable = true;
	UIDropDownMenu_AddButton(info, level);
end


-- this method builds the right-click menus
function TitanPanelRightClickMenu_PrepareCurrTrackMenu()
	-- level 2 menus
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		TitanPanelRightClickMenu_AddTitle2(UIDROPDOWNMENU_MENU_VALUE, UIDROPDOWNMENU_MENU_LEVEL);
		TitanPanelCurrTrack_GatherTokens(TitanPanelCurrTrack_BuildFactionSubMenu);
		if UIDROPDOWNMENU_MENU_VALUE == "Options" then
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_GLAMOR, TITCURRTRACK_ID, "ShowGlamorAnnounce");	
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHORT_GLAMOR, TITCURRTRACK_ID, "ShortGlamorAnnounce");	
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_ANNOUNCE, TITCURRTRACK_ID, "ShowAnnounce");	
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_LOG, TITCURRTRACK_ID, "ShowLog");	
			TitanPanelRightClickMenu_AddSpacer2(2);
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_CHANGED, TITCURRTRACK_ID, "ShowChanged");	
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_APPEND_CHANGED, TITCURRTRACK_ID, "AppendChanged");	
			TitanPanelRightClickMenu_AddSpacer2(2);
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_ALT_SUMMARY, TITCURRTRACK_ID, "ShowAltSummary");	
			TitanPanelRightClickMenu_AddToggleVar2(TITCURRTRACK_SHOW_SUMMARY, TITCURRTRACK_ID, "ShowSummary");	
			TitanPanelRightClickMenu_AddSpacer2(2);
			command = {}
			command.text = "Reset Data";
			command.value = "Reset Data";
			command.hasArrow = nil;
			command.isCheckable = nil;
			command.func = function() 
				wipe(TitanCurrTrackData);
				print("CurrencyChecker Character Data Reset!");
			end
			UIDropDownMenu_AddButton(command,2);
		end
		return;
	end
	-- level 1 menu
	TitanPanelRightClickMenu_AddTitle2(TitanPlugins[TITCURRTRACK_ID].menuText.." v"..TITCURRTRACK_VERSION);
	TitanPanelRightClickMenu_AddSpacer2();
	TitanPanelCurrTrack_GatherTokens(TitanPanelCurrTrack_BuildRightClickMenu);
	TitanPanelRightClickMenu_AddSpacer2();
	local info = {};
       	info.hasArrow = true; 
       	info.notCheckable = true;
       	info.text = "Options";
       	info.value = "Options";
	UIDropDownMenu_AddButton(info, 1);			
	TitanPanelRightClickMenu_AddSpacer2();
	info.text = "Close Menu";
	info.value = "Close Menu";
	info.hasArrow = false;
	UIDropDownMenu_AddButton(info, 1);

end

function TitanCurrTrackHeaderFactionToggle(name)
	local value = "";
	local found = false;
	local array = TitanGetVar(TITCURRTRACK_ID, "TokenHeaders");
	for index, value in ipairs(array) do
		if (value == name) then
			found = index;
		end
	end
	if(found) then 
		tremove(array,found);		
	else
		tinsert(array,name);		
	end
	TitanSetVar(TITCURRTRACK_ID, "TokenHeaders", array)
	return;
end

function TitanPanelCurrTrack_BuildRightClickMenu(name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID, parentName)
	if(not isInactive) then
		if(isHeader and not isCollapsed) then
			command = {}
			command.text = name;
			command.value = name;
			command.hasArrow = 1;
			command.keepShownOnClick = 1;
			command.checked = function() if (tContains(TitanGetVar(TITCURRTRACK_ID, "TokenHeaders"), name)) then return nil else return true end end
			command.func = function() 
				TitanCurrTrackHeaderFactionToggle(name); 
				TitanPanelButton_UpdateButton(TITCURRTRACK_ID);
			end
			UIDropDownMenu_AddButton(command);
		end
	end
end

function TitanPanelCurrTrack_BuildFactionSubMenu(name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID, parentName)
	if(parentName == UIDROPDOWNMENU_MENU_VALUE and (not isHeader)) then
		command = {}
		command.text = " |T"..icon..":24|t "..name.." ("..count..")";
		command.value = name;
		command.checked = function() if (tContains(TitanGetVar(TITCURRTRACK_ID, "TokenHeaders"), name)) then return nil else return true end end
		command.func = function() 
			TitanCurrTrackHeaderFactionToggle(name); 
			TitanPanelButton_UpdateButton(TITCURRTRACK_ID);
		end
		UIDropDownMenu_AddButton(command, UIDROPDOWNMENU_MENU_LEVEL);
	end
end

function TitanPanelCurrTrack_Refresh()
	local parentName = "";
	local changed = "";
	local all = "";
	local CharData = "";
	local name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID, cCount;
	cCount = GetCurrencyListSize();
	for index=1, cCount do 
		name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID = GetCurrencyListInfo(index);
		if (isHeader) then
			parentName = name;
		else
			if(not TITCURRTRACK_VALUES[name]) then
				TITCURRTRACK_VALUES[name] = count;
			end
			if (not tContains(TitanGetVar(TITCURRTRACK_ID, "TokenHeaders"), parentName) and
				not tContains(TitanGetVar(TITCURRTRACK_ID, "TokenHeaders"), name)) then 
				local append = "";
				if(TITCURRTRACK_VALUES[name] ~= count) then
					local diff = count - TITCURRTRACK_VALUES[name];
					if(TITCURRTRACK_TTS[name]) then
						TITCURRTRACK_TTS[name] = TITCURRTRACK_TTS[name] + diff;
					else
						TITCURRTRACK_TTS[name] = diff;
					end
					local msg = "";
					if (diff > 0) then
						changed = TitanUtils_GetGreenText("(+"..diff)..")";
						msg = "|T"..icon..":24|t "..name.." "..changed.." (Total: "..TitanUtils_GetGreenText(count)..")";
					else
						changed = TitanUtils_GetRedText("("..diff..")");
						msg = "|T"..icon..":24|t "..name.." "..changed.." (Total: "..TitanUtils_GetRedText(count)..")";
					end
					if(TitanGetVar(TITCURRTRACK_ID, "ShowGlamorAnnounce")) then
						TitanCurrTrackAlertFrame_ShowAlert(name, diff, icon, count)
					end
					TITCURRTRACK_VALUES[name] = count;
					if(TitanGetVar(TITCURRTRACK_ID, "AppendChanged") == 1) then
						if(TITCURRTRACK_TTS[name]) then
							if(TITCURRTRACK_TTS[name] > 0) then
								append = TitanUtils_GetGreenText("(+"..TITCURRTRACK_TTS[name]..")");
							elseif(TITCURRTRACK_TTS[name] < 0) then 
								append = TitanUtils_GetRedText("("..TITCURRTRACK_TTS[name]..")");
							end
						end
					end
					if(TitanGetVar(TITCURRTRACK_ID, "ShowChanged") == 1) then
						if(TITCURRTRACK_TTS[name]) then
							if(TITCURRTRACK_TTS[name] >= 0) then
								all = all.." |T"..icon..":24|t"..TitanUtils_GetGreenText(count)..append.." ";
							elseif(TITCURRTRACK_TTS[name] < 0) then
								all = all.." |T"..icon..":24|t"..TitanUtils_GetRedText(count)..append.." ";
							else
								all = all.." |T"..icon..":24|t"..count..append.." ";
							end
						else
							all = all.." |T"..icon..":24|t"..count..append.." ";
						end
					else 
						all = all.." |T"..icon..":24|t"..count..append.." ";
					end
					if(TitanGetVar(TITCURRTRACK_ID, "ShowAnnounce")) then
						UIErrorsFrame:AddMessage(msg, 2.0, 2.0, 0.0, 53, 5);
					end
					if(TitanGetVar(TITCURRTRACK_ID, "ShowLog")) then
						print(msg);
					end
				else
					if(TitanGetVar(TITCURRTRACK_ID, "AppendChanged") == 1) then
						if(TITCURRTRACK_TTS[name]) then
							if(TITCURRTRACK_TTS[name] > 0) then
								append = TitanUtils_GetGreenText("(+"..TITCURRTRACK_TTS[name]..")");
							elseif(TITCURRTRACK_TTS[name] < 0) then
								append = TitanUtils_GetRedText("("..TITCURRTRACK_TTS[name]..")");
							end
						end
					end
					if(TitanGetVar(TITCURRTRACK_ID, "ShowChanged") == 1) then
						if(TITCURRTRACK_TTS[name]) then
							if(TITCURRTRACK_TTS[name] > 0) then
								all = all.." |T"..icon..":24|t"..TitanUtils_GetGreenText(count)..append.." ";
							elseif(TITCURRTRACK_TTS[name] < 0) then
								all = all.." |T"..icon..":24|t"..TitanUtils_GetRedText(count)..append.." ";
							else
								all = all.." |T"..icon..":24|t"..count..append.." ";
							end
						else
							all = all.." |T"..icon..":24|t"..count..append.." ";
						end
					else
						all = all.." |T"..icon..":24|t"..count..append.." ";
					end
				end
			end
			if(count) then
				if(count > 0) then
					CharData = CharData.." |T"..icon..":20|t"..count.." ";
				end
			end
		end
	end
	if(all == "") then
		all = "CurrencyTracker";
	end;
	TitanCurrTrackData[TitanCurrTrack_crealm.."-"..TitanCurrTrack_cname] = CharData;
	TITCURRTRACK_BUTTON_TEXT = all;
end

function TitanPanelCurrTrack_GatherTokens(method)
		local cCount = GetCurrencyListSize();
		local done = false;
		local index = 1;
		local parentName = "";
		while(not done)do
			local name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID = GetCurrencyListInfo(index);
			local value;
			-- Normalize values
			if(isHeader) then parentName = name; end;
			method(name, isHeader, isExpanded, isUnused, isWatched, count, icon, extraCurrencyType, itemID, parentName );
			index = index+1;
			if(index>cCount) then done = true; end;
		end		
end

function TitanCurrTrackFrame_Hide() 
	TitanCurrTrackFrame:Hide(); 
end

function TitanCurrTrackFrame_Show()
	local frame = TitanCurrTrackFrame
	frame:Show();
	frame.animIn:Play();
	TitanCurrTrackFrameGlowFrame.glow.animIn:Play();
	frame.waitAndAnimOut:Stop();
	frame.waitAndAnimOut.animOut:SetStartDelay(4.5);
	frame.waitAndAnimOut:Play();
end


-- [[ TitanCurrTrackAlertFrame ]] --
function TitanCurrTrackAlertFrame_OnLoad (self)
	self:RegisterForClicks("LeftButtonUp");
end



function TitanCurrTrackAlertFrame_FixAnchors ()
	-- Temporary (here's hoping) workaround so that achievement alerts are anchored to loot roll windows. Eventually we want one system to handle placement for both alerts.
	if ( not TitanCurrTrackAlertFrame1 ) then
		-- We haven't displayed any achievement alerts yet, so there's nothing to reanchor (read: this got called by LootFrame.lua)
		return;
	end
	
	for i=NUM_GROUP_LOOT_FRAMES, 1, -1  do
		local frame = _G["GroupLootFrame"..i];
		if ( frame and frame:IsShown() ) then
			TitanCurrTrackAlertFrame1:SetPoint("BOTTOM", frame, "TOP", 0, 10);
			return;
		end
	end
	
	TitanCurrTrackAlertFrame1:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 128);
end

function TitanCurrTrackAlertFrame_ShowAlert(name, diff, icon, points)
	local frame = TitanCurrTrackAlertFrame_GetAlertFrame(points);
	if ( not frame ) then
		-- We ran out of frames! Bail!
		return;
	end
	
	--local _, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch = GetAchievementInfo(achievementID);

	local frameName = frame:GetName();
	local displayName = _G[frameName.."Name"];
	local displayNameTop = _G[frameName.."IconNameTop"];
	local displayNameBottom = _G[frameName.."IconNameBottom"];
	local shieldPoints = _G[frameName.."ShieldPoints"];
	local shieldIcon = _G[frameName.."ShieldIcon"];
	local BackGround = _G[frameName.."Background"];
	local Unlocked = _G[frameName.."Unlocked"];
	
	if(TitanGetVar(TITCURRTRACK_ID, "ShortGlamorAnnounce")) then
		frame:SetWidth(44);
		frame.glow:Hide();
		frame.shine:Hide();
		BackGround:Hide();
		Unlocked:Hide();
		displayName:SetText("");
		shieldPoints:Hide();
		shieldIcon:Hide();
		if (diff > 0) then
			displayNameTop:SetText(TitanUtils_GetGreenText(" (+"..diff..")"));
		else
			displayNameTop:SetText(TitanUtils_GetRedText(" ("..diff..")"));
		end
		displayNameBottom:SetText(points);
	else
		frame:SetWidth(300);
		frame.glow:Show();
		frame.shine:Show();
		BackGround:Show();
		Unlocked:Show();
		shieldPoints:Show();
		shieldIcon:Show();
		displayNameTop:SetText("");
		displayNameBottom:SetText("");
		if (diff > 0) then
			displayName:SetText(name..TitanUtils_GetGreenText(" (+"..diff..")"));
		else
			displayName:SetText(name..TitanUtils_GetRedText(" ("..diff..")"));
		end
	end
	MyAchievementShield_SetPoints(points, shieldPoints, GameFontNormal, GameFontNormalSmall);
	
	if (diff > 0) then
		frame.glow:SetVertexColor(0,0.5,0);
		frame.shine:SetVertexColor(0,0.5,0);
		frame.unlocked:SetText("Token Earned");
	else
		frame.glow:SetVertexColor(0.5,0,0);
		frame.shine:SetVertexColor(0.5,0,0);
		frame.unlocked:SetText("Token Lost");
	end
	
	if ( points == 0 ) then
		shieldIcon:SetTexture([[Interface\AchievementFrame\UI-Achievement-Shields-NoPoints]]);
	else
		shieldIcon:SetTexture([[Interface\AchievementFrame\UI-Achievement-Shields]]);
	end
	
	_G[frameName.."IconTexture"]:SetTexture(icon);
	
	frame.id = achievementID;
	
	AlertFrame_AnimateIn(frame);
	
	AlertFrame_FixAnchors();
end

function TitanCurrTrackAlertFrame_GetAlertFrame(diff)
	local name, frame, previousFrame;
	for i=1, 6 do
		name = "TitanCurrTrackAlertFrame"..i;
		frame = _G[name];
		if ( frame ) then
			if ( not frame:IsShown() ) then
				return frame;
			end
		else
			frame = CreateFrame("Button", name, UIParent, "TitanCurrTrackAlertFrameTemplate");
			if ( not previousFrame ) then
				frame:SetPoint("CENTER", UIErrorsFrame, "BOTTOM", 0, 0);
			else
				frame:SetPoint("TOP", previousFrame, "BOTTOM", 0, -10);
			end
			return frame;
		end
		previousFrame = frame;
	end
	return nil;
end

function TitanCurrTrackAlertFrame_OnClick (self)
	local id = self.id;
	if ( not id ) then
		return;
	end
	
	CloseAllWindows();
	ShowUIPanel(AchievementFrame);
	
	local _, _, _, achCompleted = GetAchievementInfo(id);
	if ( achCompleted and (ACHIEVEMENTUI_SELECTEDFILTER == AchievementFrameFilters[ACHIEVEMENT_FILTER_INCOMPLETE].func) ) then
		AchievementFrame_SetFilter(ACHIEVEMENT_FILTER_ALL);
	elseif ( (not achCompleted) and (ACHIEVEMENTUI_SELECTEDFILTER == AchievementFrameFilters[ACHIEVEMENT_FILTER_COMPLETE].func) ) then
		AchievementFrame_SetFilter(ACHIEVEMENT_FILTER_ALL);
	end
	
	AchievementFrame_SelectAchievement(id)
end

function TitanCurrTrackAlertFrame_OnHide (self)
	AlertFrame_FixAnchors();
end

function MyAchievementShield_OnLoad (self)
	self.Desaturate = AchievementShield_Desaturate;
	self.Saturate = AchievementShield_Saturate;
end

function MyAchievementShield_SetPoints(points, pointString, normalFont, smallFont)
	if ( points == 0 ) then
		pointString:SetText("");
		return;
	end
	if ( points <= 100 ) then
		pointString:SetFontObject(normalFont);
	else
		pointString:SetFontObject(smallFont);
	end
	pointString:SetText(points);
end
